@@ -1,6 +1,7 @@
 from .blocks import ParameterBlock, CompartmentBlock, ObservableBlock
 from .blocks import SpeciesBlock, MoleculeTypeBlock
 from .blocks import FunctionBlock, RuleBlock
+from .blocks import EnergyPatternBlock
 
 from .pattern import Pattern, Molecule, Component
 
@@ -557,7 +558,9 @@ class RuleBlockXML(XMLObj):
                     )
                 rate_constants = [self.resolve_ratelaw(rule["RateLaw"])]
                 rule_modifier = self.get_rule_mod(rule)
-                operations = self.get_operations(rule["ListOfOperations"])
+                if rule["ListOfOperations"] is not None:
+                    if len(rule["ListOfOperations"]) > 0:
+                        operations = self.get_operations(rule["ListOfOperations"])
 
                 block.add_rule(
                     name,
@@ -687,11 +690,12 @@ class RuleBlockXML(XMLObj):
         # TODO: create working rule mods class
         rule_mod = RuleMod()
         list_ops = xml["ListOfOperations"]
+        if list_ops is None:
+            return None
         # determine which rule mod is being used, if any
         if "Delete" in list_ops:
             del_op = list_ops["Delete"]
             # check if modifier was called or automatic
-            # import IPython; IPython.embed()
             if not isinstance(del_op, list):
                 mod_call = del_op["@DeleteMolecules"]
                 if mod_call == "1":
@@ -749,6 +753,39 @@ class RuleBlockXML(XMLObj):
         return rule_mod
 
 
+class EnergyPatternBlockXML(XMLObj):
+    """
+    EnergyPatternBlock XML parser, derived from XMLObj. Creates
+    an EnergyPatternBlock that contains the energy patterns parsed.
+    """
+
+    def __init__(self, xml):
+        super().__init__(xml)
+
+    def parse_xml(self, xml):
+        block = EnergyPatternBlock()
+
+        if isinstance(xml, list):
+            for b in xml:
+                # get id & expression
+                epid = b["@id"]
+                expr = b["@expression"]
+                # get pattern
+                pattern_node = b["Pattern"]
+                pattern = PatternXML(pattern_node).parsed_obj
+                block.add_energy_pattern(epid, pattern, expr)
+        else:
+            # get id & expression
+            epid = xml["@id"]
+            expr = xml["@expression"]
+            # get pattern
+            pattern_node = xml["Pattern"]
+            pattern = PatternXML(pattern_node).parsed_obj
+            block.add_energy_pattern(epid, pattern, expr)
+
+        return block
+
+
 class Operation:
     """
     To be used for parsing & storing ListOfOperations information.
